// clipboard
// © jonas.sundstrom@kirilla.com

#define DEBUG 0
#include <Debug.h>

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include <Application.h>
#include <Roster.h>
#include <String.h>
#include <File.h>
#include <Clipboard.h>
#include <FindDirectory.h>
#include <Path.h>
#include <string.h>

#include "clipboard.h"

int	exitvalue = EXIT_SUCCESS;;

int main()
{
	ClipboardApp the_app;
	the_app.Run();

	return (exitvalue);
}

ClipboardApp::ClipboardApp	(void)
 : BApplication	("application/x-vnd.Kirilla-clipboard")
{
	got_arguments	=	false;	
}

ClipboardApp::~ClipboardApp	(void)
{

}

void 
ClipboardApp::ArgvReceived	(int32 argc, char **argv)
{
	BString option	=	argv[1];
	BString file	=	argv[2];
	
	if (option.IFindFirst("--help") == 0)
	{
		Help();
		
		got_arguments =	true;
		return;
	}
	
	if (option.IFindFirst("--empty") == 0)
	{
		Empty();
	
		got_arguments =	true;
		return;
	}
	
	if (option.IFindFirst("--stdin") == 0)
	{
		Make(NULL);
		
		got_arguments =	true;
		return;
	}
	
	if (option.IFindFirst("--string=") == 0)
	{
		option.IReplaceFirst("--string=", "");
		Make(option.String());

		got_arguments =	true;
		return;
	}
	
	if (option.IFindFirst("--print") == 0)
	{
		Print(false);

		got_arguments =	true;
		return;
	}

	if (option.IFindFirst("--stdout") == 0)
	{
		Print(false);

		got_arguments =	true;
		return;
	}

	if (option.IFindFirst("--debug") == 0)
	{
		Print(true);

		got_arguments =	true;
		return;
	}

	if (option.IFindFirst("--load") == 0)
	{
		if (argv[2] == NULL)
			Load(NULL);
		else
		{
			file.IReplaceFirst("--file=", "");
			Load(file.String());
		}

		got_arguments =	true;	
		return;
	}

	if (option.IFindFirst("--save") == 0)
	{
		if (argv[2] == NULL)
			Save(NULL);
		else
		{
			file.IReplaceFirst("--file=", "");
			Save(file.String());
		}
		
		got_arguments =	true;
		return;
	}
}

void
ClipboardApp::ReadyToRun	(void)
{
	if (got_arguments == false)
		Help();

	PostMessage(B_QUIT_REQUESTED);
}


void
ClipboardApp::Help	(void)
{
	printf(	"clipboard 2.0, by jonas.sundstrom@kirilla.com\n\n"

			"Syntax: clipboard  --option  [--file=path]\n\n"

			"Options:\n"

			"--save            Save clipboard (flattened BMessage)\n"
			"--load            Load clipboard (flattened BMessage)\n\n"

			"--file=path       Unless a file is specified, --save/--load\n"
			"                  will default to a file named 'clipboard-clip',\n"
			"                  in your settings directory\n\n"

			"--stdin           Load clipboard from standard input\n"
			"--string=\"xyz\"    Load clipboard from the given string\n\n"

			"--print           Print clipboard to standard output\n"
			"--stdout          Same as --print\n"
			"--debug           Print clipboard message to stdout\n\n"

			"--empty           Remove all contents from clipboard\n\n"

			"--help            Display this marvelous help text\n\n");
}

void
ClipboardApp::Empty	()
{
	status_t	status	=	B_OK;
	
	if (! be_clipboard->Lock())
	{
		printf("Error: could not lock clipboard.\n");
		exitvalue = EXIT_FAILURE;
		return;
	}

	be_clipboard->Clear();
	status = be_clipboard->Commit();
		
	if (status != B_OK)
	{
		be_clipboard->Unlock();
		exitvalue = EXIT_FAILURE;
		return;
	}
	
	be_clipboard->Unlock();

	printf("Success: clipboard emptied.\n");
}

void
ClipboardApp::Load	(const char * a_path)
{
	status_t	status	=	B_OK;
	
	// create BFile
	BFile	file;
	
	if (a_path == NULL)
	{
		// default location
		BPath path;
		find_directory(B_USER_SETTINGS_DIRECTORY, & path);
		
		path.SetTo(path.Path(), "clipboard-clip");
		
		status	=	file.SetTo(path.Path(), B_READ_ONLY);
	}
	else
	{
		// user-specified path
		status	=	file.SetTo(a_path, B_READ_ONLY);
	}
	
	if (status != B_OK)
	{
		Empty();
	
		printf("Error: ");
		if (status == B_PERMISSION_DENIED)
			printf("permission denied.\n");
		else if (status == B_ENTRY_NOT_FOUND)
			printf("file does not exist.\n");
		else
			printf("%s\n", strerror(status));
		
		exitvalue = EXIT_FAILURE;
		return;
	}
	
	// get BMesage from clip
	BMessage	filed_clip;
	
	status	=	filed_clip.Unflatten(& file);
	
	if (status != B_OK)
	{
		Empty();

		printf("Error: %s\n", strerror(status));
		exitvalue = EXIT_FAILURE;
		return;
	}
	

	// stuff clip into clipboard
	
	BMessage	*	clip	=	NULL;
	if (! be_clipboard->Lock())
	{
		printf("Error: could not lock clipboard.\n");
		exitvalue = EXIT_FAILURE;
		return;
	}

	be_clipboard->Clear();
	
	clip = be_clipboard->Data();
	
	*clip = filed_clip;
	
	status = be_clipboard->Commit();
	
	if (status != B_OK)
	{
		be_clipboard->Unlock();

		exitvalue = EXIT_FAILURE;
		return;
	}

	be_clipboard->Unlock();

	printf("Success: clip loaded.\n");
}

void
ClipboardApp::Make	(const char * a_string)
{
	status_t	status	=	B_OK;
	BString		std_input;
	
	if (a_string == NULL)
	{
		// read from standard input
		char c;
		while ((c = cin.get()) != EOF)
		{
			std_input	+=	c;
		}
		
		a_string = std_input.String();
	}
	
	// get clipboard BMessage
	
	BMessage	*	clip	=	NULL;
	
	if (be_clipboard->Lock())
	{
		be_clipboard->Clear();
		
		clip = be_clipboard->Data();

		// add data to clipboard
			
		clip->AddData("text/plain", B_MIME_TYPE, a_string, strlen(a_string));			
		
		status = be_clipboard->Commit();
		
		if (status != B_OK)
		{
			be_clipboard->Unlock();
			exitvalue = EXIT_FAILURE;
			return;
		}
	
		be_clipboard->Unlock();
	}
	else
	{
		printf("Error: could not lock clipboard.\n");
		exitvalue = EXIT_FAILURE;
		return;
	}
	
	printf("Success: data added to clipboard.\n");
}

void
ClipboardApp::Print	(bool a_debug)
{
	// get clip & print contents 
	// (or the entire clip, in case of --debug)
	BMessage	*	clip;
	
	if (! be_clipboard->Lock())
	{
		printf("Error: could not lock clipboard.\n");
		exitvalue = EXIT_FAILURE;
		return;
	}

	clip = be_clipboard->Data();

	if (a_debug)
		clip->PrintToStream();
	else
	{
		const char *	text;
		int32 			text_len;
		BString			out_string;
		
		clip->	FindData("text/plain", B_MIME_TYPE, (const void **)&text, &text_len);
		
		if (text != NULL)
			out_string.SetTo(text, text_len);
		
		printf("%s\n", out_string.String());
	}	

	be_clipboard->Unlock();
}

void
ClipboardApp::Save	(const char * a_path)
{
	status_t	status	=	B_OK;
	
	// create BFile
	BFile	file;
	
	if (a_path == NULL)
	{
		// default location
		BPath path;
		find_directory(B_USER_SETTINGS_DIRECTORY, & path);
		
		path.SetTo(path.Path(), "clipboard-clip");
		
		status	=	file.SetTo(path.Path(), B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	}
	else
	{
		// user-specified path
		status	=	file.SetTo(a_path, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	}
	
	if (status != B_OK)
	{
		if (status == B_PERMISSION_DENIED)
			printf("Error: permission denied.\n");
			
		if (status == B_BAD_VALUE)
			printf("Error: path does not exist.\n");
		
		printf("(%s)\n", strerror(status));
		
		exitvalue = EXIT_FAILURE;
		return;
	}
	
	// get clip
	if (! be_clipboard->Lock())
	{
		printf("Error: could not lock clipboard.\n");
		exitvalue = EXIT_FAILURE;
		return;
	}
		
	BMessage *	clip;	
	clip = be_clipboard->Data();
	
	be_clipboard->Unlock();
	
	// flatten clip to file
	status	=	clip->	Flatten(& file);
	
	if (status != B_OK)
	{
		if (status == B_NO_MEMORY)
			printf("Error: out of space\n");
			
		if (status == B_BAD_VALUE)
			printf("Error: unknown error.\n");
			
		printf("(%s)\n", strerror(status));
		
		exitvalue = EXIT_FAILURE;
		return;
	}
	
	printf("Success: clip saved.\n");
}

